/*

Actuate Client Example

This sample demonstrates Administrate request including
the following operations:

	CreateUser
	CreateRole
	UpdateUser
	CreateFolder

Usage:
     Administrate [Common options]

Example:
     Administrate -h http://localhost:1111 -v volume


Common options are:
     -h hostname    SOAP endpoint, default 'http://localhost:8000'
     -u username    specify username, default 'Administrator'
     -p password    specify password, default ''
     -v volume      specify volume name, default ''

*/

using System;
using System.Web.Services.Protocols;
using Server_Proxy.localhost;

namespace Actuate
{
	class ClassMain
	{
		[STAThread]
		static void Main(string[] args)
		{
			string l_url;
			string l_userName;
			string l_password;
            string l_volume;
			ActuateAPI l_proxy;

            Usage(args, out l_url, out l_userName, out l_password, out l_volume);
			Login(out l_proxy, l_url, l_userName, l_password, l_volume);

			//	Prepare CreateUser operation
			CreateUser l_createUser = new CreateUser();
			l_createUser.IgnoreDup = true;
			l_createUser.IgnoreDupSpecified = true;
			l_createUser.User = new User();
			l_createUser.User.Name = "Tester";
			l_createUser.User.Password = "Tester";
			l_createUser.User.HomeFolder = "Tester"; // home folder will be created automatically
			l_createUser.User.ViewPreference = UserViewPreference.DHTML;
			l_createUser.User.ViewPreferenceSpecified = true;
			l_createUser.User.SendEmailForSuccess = true;
			l_createUser.User.SendNoticeForSuccessSpecified = true;

			AdminOperation l_createUserOpt = new AdminOperation();
			l_createUserOpt.Item = l_createUser;

			//	Prepare CreateRole operation
			CreateRole l_createRole = new CreateRole();
			l_createRole.IgnoreDup = true;
			l_createRole.IgnoreDupSpecified = true;
			l_createRole.Role = new Role();
			l_createRole.Role.Name = "Tester Role";
			l_createRole.Role.Description = "This is the role for the tester";

			AdminOperation l_createRoleOpt = new AdminOperation();
			l_createRoleOpt.Item = l_createRole;

			//	Then update user's properties
			UpdateUser l_updateUser = new UpdateUser();
			l_updateUser.IgnoreMissing = true;
			l_updateUser.IgnoreMissingSpecified = true;
			l_updateUser.IgnoreDup = true;
			l_updateUser.IgnoreDupSpecified = true;
			l_updateUser.SkipPermissionError = true;
			l_updateUser.SkipPermissionErrorSpecified = true;
			l_updateUser.Item = "Tester";
            l_updateUser.ItemElementName = ItemChoiceType23.Name;

			// assign role
			UpdateUserOperation l_assignRole = new UpdateUserOperation();
            StringArray l_roleNames = new StringArray(); 
			l_roleNames.String = new string[1];
			l_roleNames.String[0] = "Tester Role";
			l_assignRole.Item = l_roleNames;
            l_assignRole.ItemElementName = ItemChoiceType22.AssignRolesByName;

			l_updateUser.UpdateUserOperationGroup = new UpdateUserOperation[1];
			l_updateUser.UpdateUserOperationGroup[0] = l_assignRole;

			AdminOperation l_updateUserOpt = new AdminOperation();
			l_updateUserOpt.Item = l_updateUser;

            //	Prepare CreateFolder operation. Create 1 folder with 5 subfolders
			// using Transaction
			Transaction l_createFolderTransOpts = new Transaction();
	            l_createFolderTransOpts.TransactionOperation = new TransactionOperation[6];

			// create parent folder
			CreateFolder l_createFolder = new CreateFolder();
			l_createFolder.IgnoreDup = true;
			l_createFolder.IgnoreDupSpecified = true;
			l_createFolder.FolderName = "Report";
			l_createFolder.Item = "/";
            l_createFolder.ItemElementName = ItemChoiceType38.WorkingFolderName;
			l_createFolder.Description = "1st level example folders";

			l_createFolderTransOpts.TransactionOperation[0] = new TransactionOperation();
			l_createFolderTransOpts.TransactionOperation[0].Item = l_createFolder;

			//	create subfolders
			for (int i = 0; i < 5; i++)
			{
				CreateFolder l_tmpCreateFolder = new CreateFolder();
				l_tmpCreateFolder.IgnoreDup = true;
				l_tmpCreateFolder.IgnoreDupSpecified = true;
				l_tmpCreateFolder.Description = "2nd level example folders";
				l_tmpCreateFolder.FolderName = "subFolder" + i;
				l_tmpCreateFolder.Item = "/Report";
                l_tmpCreateFolder.ItemElementName = ItemChoiceType38.WorkingFolderName;

	      	      l_createFolderTransOpts.TransactionOperation[i + 1] = new TransactionOperation();
      	      	l_createFolderTransOpts.TransactionOperation[i + 1].Item = l_tmpCreateFolder;
			}

			AdminOperation l_createFoldersOpt = new AdminOperation();
			l_createFoldersOpt.Item = l_createFolderTransOpts;
			
			//	Assemble the administrate request.
			AdminOperation[] l_adminRequest = new AdminOperation[4];
			l_adminRequest[0] = l_createUserOpt;
			l_adminRequest[1] = l_createRoleOpt;
			l_adminRequest[2] = l_updateUserOpt;
            l_adminRequest[3] = l_createFoldersOpt;
			
			try 
			{
				l_proxy.administrate(l_adminRequest);
			}
			catch(Exception e)
			{	
				PrintExceptionDetails(e);
				return;
			}
		}

		///////////////////////////////////////////////////////////////////////
		//	Command line usage
		///////////////////////////////////////////////////////////////////////
		static void Usage(string[] args, out string url, out string userName, out string password, out string volume)
		{
			string l_url = "http://localhost:8000";
			string l_userName = "administrator";
			string l_password = "";
            string l_volume = "";

			for(int i = 0; i < args.Length; i++)
			{
				if(args[i].ToLower().CompareTo("-h") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_url = args[i + 1];
					}
				}
				else if(args[i].ToLower().CompareTo("-u") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_userName = args[i + 1];
					}
				}
				else if(args[i].ToLower().CompareTo("-p") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_password = args[i + 1];
					}
				}
                else if (args[i].ToLower().CompareTo("-v") == 0)
                {
                    if (!args[i + 1].StartsWith("-"))
                    {
                        l_volume = args[i + 1];
                    }
                }
            }

			url = l_url;
			userName = l_userName;
			password = l_password;
            volume = l_volume;
        }

		///////////////////////////////////////////////////////////////////////
		//	Server login
		///////////////////////////////////////////////////////////////////////
        static void Login(out ActuateAPI proxy, string url, string userName, string password, string volume)
		{
			//	Create an instance of server proxy. Construct an empty header.
			proxy = new ActuateAPI();
			proxy.Url = url;
			proxy.HeaderValue = new Header();
            proxy.HeaderValue.TargetVolume = volume;

			//	Prepare Login reqest parameters
			Server_Proxy.localhost.Login l_LoginReq = new Server_Proxy.localhost.Login();
			l_LoginReq.User = userName;
			l_LoginReq.Password = password;
//            l_LoginReq.Domain = l_volume;

			//	Send Login reqest, handle SOAP exception if any
			LoginResponse l_loginRes;
			try
			{
				l_loginRes = proxy.login(l_LoginReq);
			}
			catch(Exception e)
			{
				PrintExceptionDetails(e);
				return;
			}

			//	Store authentication id
			proxy.HeaderValue.AuthId = l_loginRes.AuthId;
            proxy.HeaderValue.TargetVolume = volume;
		}

		///////////////////////////////////////////////////////////////////////
		//	SOAP exception parsing and output
		///////////////////////////////////////////////////////////////////////
		static void PrintExceptionDetails(Exception e)
		{
			if(e is SoapException)
			{
				SoapException l_e = (SoapException)e;
				Console.WriteLine("# SOAP Fault received: ");
				for(int i = 0; i < l_e.Detail.ChildNodes.Count; i++)
				{
					if(l_e.Detail.ChildNodes.Item(i).Name[0] != '#')
					{
						if(l_e.Detail.ChildNodes.Item(i).Name == "Description")
						{
							for(int j = 0; j < l_e.Detail.ChildNodes.Item(i).ChildNodes.Count; j ++)
							{
								if(l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).Name[0] != '#')
								{
									Console.WriteLine(l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).Name + ": "
										+ l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).InnerText);
								}
							}
						}
						else
						{
							Console.WriteLine(l_e.Detail.ChildNodes.Item(i).Name + ": " + l_e.Detail.ChildNodes.Item(i).InnerText);
						}
					}
				}
			}
			else
			{
				Console.WriteLine("# Non-SOAP exception:");
				Console.WriteLine(e.ToString());
			}
		}
	}
}
