/*

Actuate Client Example

This sample demonstrates how to download a file as a MIME
attachment assuming that "/Report/SampleBIRTReport.rptdesign" file
already exists in encyclopedia. This sample uses ActuateAPIEx
class (extension) implemented in Server Proxy project ("Server Proxy.dll").

Usage:
     DownloadFile_SOAP_Attachment [Common options]

Example:
     DownloadFile_SOAP_Attachment -h http://localhost:1111  -v volume 


Common options are:
     -h hostname    SOAP endpoint, default 'http://localhost:8000'
     -u username    specify username, default 'Administrator'
     -p password    specify password, default ''
     -v volume      specify volume name, default ''
     
*/

using System;
using System.IO;
using System.Web.Services.Protocols;
using Server_Proxy;
using Server_Proxy.localhost;

namespace Actuate
{
	class MainClass
	{
		[STAThread]
		static void Main(string[] args)
		{
			string l_url;
			string l_userName;
			string l_password;
			string l_volume;
			ActuateAPI l_proxy;

			Usage(args, out l_url, out l_userName, out l_password, out l_volume);
			Login(out l_proxy, l_url, l_userName, l_password, l_volume);

			// Create an instance of extended proxy
			ActuateAPIEx l_proxyEx= new ActuateAPIEx();
			l_proxyEx.Url = l_proxy.Url;
			l_proxyEx.HeaderValue = new Header();
			l_proxyEx.HeaderValue.AuthId = l_proxy.HeaderValue.AuthId;
			l_proxyEx.HeaderValue.TargetVolume = l_volume;

			// Prepare DownlodFile request
			DownloadFile l_req = new DownloadFile();
            l_req.Item = "/Report/SampleBIRTReport.rptdesign";
            l_req.ItemElementName = ItemChoiceType52.FileName;
			l_req.DownloadEmbedded = false;

			DownloadFileResponse l_res;
			try
			{
				l_res = l_proxyEx.downloadFile(l_req);
			}
			catch(Exception e)
			{
				PrintExceptionDetails(e);
				return;
			}

			// Move downloaded file
            System.IO.File.Move(l_proxyEx.DownloadFileHeader.TargetFileName, "../../download/SampleBIRTReport.rptdesign");
		}

		///////////////////////////////////////////////////////////////////////
		//	Command line usage
		///////////////////////////////////////////////////////////////////////
		static void Usage(string[] args, out string url, out string userName, out string password, out string volume)
		{
			string l_url = "http://localhost:8000";
			string l_userName = "administrator";
			string l_password = "";
			string l_volume = "";

			for(int i = 0; i < args.Length; i++)
			{
				if(args[i].ToLower().CompareTo("-h") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_url = args[i + 1];
					}
				}
				else if(args[i].ToLower().CompareTo("-u") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_userName = args[i + 1];
					}
				}
				else if(args[i].ToLower().CompareTo("-p") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_password = args[i + 1];
					}
				}
				else if (args[i].ToLower().CompareTo("-v") == 0)
				{
				    if (!args[i + 1].StartsWith("-"))
				    {
				        l_volume = args[i + 1];
				    }
				}
			}

			url = l_url;
			userName = l_userName;
			password = l_password;
			volume = l_volume;
		}

		///////////////////////////////////////////////////////////////////////
		//	Server login
		///////////////////////////////////////////////////////////////////////
		static void Login(out ActuateAPI proxy, string url, string userName, string password, string volume)
		{
			//	Create an instance of server proxy. Construct an empty header.
			proxy = new ActuateAPI();
			proxy.Url = url;
			proxy.HeaderValue = new Header();
			proxy.HeaderValue.TargetVolume = volume;

			//	Prepare Login reqest parameters
			Server_Proxy.localhost.Login l_LoginReq = new Server_Proxy.localhost.Login();
			l_LoginReq.User = userName;
			l_LoginReq.Password = password;

			//	Send Login reqest, handle SOAP exception if any
			LoginResponse l_loginRes;
			try
			{
				l_loginRes = proxy.login(l_LoginReq);
			}
			catch(Exception e)
			{
				PrintExceptionDetails(e);
				return;
			}

			//	Store authentication id
			proxy.HeaderValue.AuthId = l_loginRes.AuthId;
		}

		///////////////////////////////////////////////////////////////////////
		//	SOAP exception parsing and output
		///////////////////////////////////////////////////////////////////////
		static void PrintExceptionDetails(Exception e)
		{
			if(e is SoapException)
			{
				SoapException l_e = (SoapException)e;
				Console.WriteLine("# SOAP Fault received: ");
				for(int i = 0; i < l_e.Detail.ChildNodes.Count; i++)
				{
					if(l_e.Detail.ChildNodes.Item(i).Name[0] != '#')
					{
						if(l_e.Detail.ChildNodes.Item(i).Name == "Description")
						{
							for(int j = 0; j < l_e.Detail.ChildNodes.Item(i).ChildNodes.Count; j ++)
							{
								if(l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).Name[0] != '#')
								{
									Console.WriteLine(l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).Name + ": "
										+ l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).InnerText);
								}
							}
						}
						else
						{
							Console.WriteLine(l_e.Detail.ChildNodes.Item(i).Name + ": " + l_e.Detail.ChildNodes.Item(i).InnerText);
						}
					}
				}
			}
			else
			{
				Console.WriteLine("# Non-SOAP exception:");
				Console.WriteLine(e.ToString());
			}
		}
	}
}
