/*

Actuate Client Example

This sample demonstrates GetFolderItems request. The sample
retrieves all items (files/folders) in "/Report" folder
created in Administrate sample. The sample also demonstrates
using of fetching.

Usage:
     GetFolderItems [Common options]

Example:
     GetFolderItems -h http://localhost:1111 -v volume

Common options are:
     -h hostname    SOAP endpoint, default 'http://localhost:8000'
     -u username    specify username, default 'Administrator'
     -p password    specify password, default ''
     -v volume      specify volume name, default ''

*/

using System;
using System.Web.Services.Protocols;
using Server_Proxy.localhost;

namespace Actuate
{
	class MainClass
	{
		[STAThread]
		static void Main(string[] args)
		{
			string l_url;
			string l_userName;
			string l_password;
			string l_volume;
			ActuateAPI l_proxy;

			Usage(args, out l_url, out l_userName, out l_password, out l_volume);
			Login(out l_proxy, l_url, l_userName, l_password, l_volume);

			// Prepare GetFolderItems request
			Server_Proxy.localhost.GetFolderItems l_req = new Server_Proxy.localhost.GetFolderItems();
            l_req.ItemElementName = Server_Proxy.localhost.ItemChoiceType54.FolderName;
			l_req.Item = "/Report";
			l_req.LatestVersionOnly = true;
			l_req.LatestVersionOnlySpecified = true;
			l_req.ResultDef = new string[3];
			l_req.ResultDef[0] = "FileType";
			l_req.ResultDef[1] = "Description";
			l_req.ResultDef[2] = "Owner";

			//	Prepare the search critaria: set fetch size = 2. So server will
			// return 2 items a time in loop until all itemes are listed.
			l_req.Search = new FileSearch();
			FileCondition l_searchCondition = new FileCondition();
			l_searchCondition.Field = FileField.Name;
			l_searchCondition.Match = "*";
			l_req.Search.Item = l_searchCondition;
			l_req.Search.FetchSize = 2;
			l_req.Search.FetchSizeSpecified = true;
			l_req.Search.CountLimit = -1; // count all items
			l_req.Search.CountLimitSpecified = true;

			GetFolderItemsResponse l_res;
			int count = 0;

			//	Loop to retrieve all the items
			do 
			{
				try
				{
					l_res = l_proxy.getFolderItems(l_req);
				}
				catch(Exception e)
				{
					PrintExceptionDetails(e);
					return;
				}

				if(count == 0)
				{
					Console.WriteLine("Total Count: " + l_res.TotalCount);
				}

				Console.WriteLine();

				for(int i = 0; i < l_res.ItemList.Length; i++)
				{
					Console.WriteLine("Item " + (i + count) + " Id:" + l_res.ItemList[i].Id);
					Console.WriteLine("Item " + (i + count) + " Name:" + l_res.ItemList[i].Name);
					Console.WriteLine("Item " + (i + count) + " Owner:" + l_res.ItemList[i].Owner);
					Console.WriteLine("Item " + (i + count) + " Description:" + l_res.ItemList[i].Description);
					Console.WriteLine("Item " + (i + count) + " File Type:" + l_res.ItemList[i].FileType);
				}

				count += l_res.ItemList.Length;

				//	Reusing the fetch handle
				l_req.Search.FetchHandle = l_res.FetchHandle;
			} while(l_res.FetchHandle != null); // FetchHandle will be empty if there is no more items left
		}

		///////////////////////////////////////////////////////////////////////
		//	Command line usage
		///////////////////////////////////////////////////////////////////////
		static void Usage(string[] args, out string url, out string userName, out string password, out string volume)
		{
			string l_url = "http://localhost:8000";
			string l_userName = "administrator";
			string l_password = "";
			string l_volume = "";

			for(int i = 0; i < args.Length; i++)
			{
				if(args[i].ToLower().CompareTo("-h") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_url = args[i + 1];
					}
				}
				else if(args[i].ToLower().CompareTo("-u") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_userName = args[i + 1];
					}
				}
				else if(args[i].ToLower().CompareTo("-p") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_password = args[i + 1];
					}
				}
				else if (args[i].ToLower().CompareTo("-v") == 0)
				{
				    if (!args[i + 1].StartsWith("-"))
				    {
				        l_volume = args[i + 1];
				    }
				}
			}

			url = l_url;
			userName = l_userName;
			password = l_password;
			volume = l_volume;
		}

		///////////////////////////////////////////////////////////////////////
		//	Server login
		///////////////////////////////////////////////////////////////////////
		static void Login(out ActuateAPI proxy, string url, string userName, string password, string volume)
		{
			//	Create an instance of server proxy. Construct an empty header.
			proxy = new ActuateAPI();
			proxy.Url = url;
			proxy.HeaderValue = new Header();
			proxy.HeaderValue.TargetVolume = volume;

			//	Prepare Login reqest parameters
			Server_Proxy.localhost.Login l_LoginReq = new Server_Proxy.localhost.Login();
			l_LoginReq.User = userName;
			l_LoginReq.Password = password;

			//	Send Login reqest, handle SOAP exception if any
			LoginResponse l_loginRes;
			try
			{
				l_loginRes = proxy.login(l_LoginReq);
			}
			catch(Exception e)
			{
				PrintExceptionDetails(e);
				return;
			}

			//	Store authentication id
			proxy.HeaderValue.AuthId = l_loginRes.AuthId;
		}

		///////////////////////////////////////////////////////////////////////
		//	SOAP exception parsing and output
		///////////////////////////////////////////////////////////////////////
		static void PrintExceptionDetails(Exception e)
		{
			if(e is SoapException)
			{
				SoapException l_e = (SoapException)e;
				Console.WriteLine("# SOAP Fault received: ");
				for(int i = 0; i < l_e.Detail.ChildNodes.Count; i++)
				{
					if(l_e.Detail.ChildNodes.Item(i).Name[0] != '#')
					{
						if(l_e.Detail.ChildNodes.Item(i).Name == "Description")
						{
							for(int j = 0; j < l_e.Detail.ChildNodes.Item(i).ChildNodes.Count; j ++)
							{
								if(l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).Name[0] != '#')
								{
									Console.WriteLine(l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).Name + ": "
										+ l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).InnerText);
								}
							}
						}
						else
						{
							Console.WriteLine(l_e.Detail.ChildNodes.Item(i).Name + ": " + l_e.Detail.ChildNodes.Item(i).InnerText);
						}
					}
				}
			}
			else
			{
				Console.WriteLine("# Non-SOAP exception:");
				Console.WriteLine(e.ToString());
			}
		}
	}
}
