/*

Actuate Client Example

This sample demonstrates server Login message.

Usage:
     Login [Common options]

Example:
     Login -h http://localhost:1111 -v volume


Common options are:
     -h hostname    SOAP endpoint, default 'http://localhost:8000'
     -u username    specify username, default 'Administrator'
     -p password    specify password, default ''
     -v volume      specify volume name, default ''
     
*/

using System;
using System.Web.Services.Protocols;
using Server_Proxy.localhost;

namespace Actuate
{
	class MainClass
	{
		[STAThread]
		static void Main(string[] args)
		{
			string l_url;
			string l_userName;
			string l_password;
			string l_volume;
			
			Usage(args, out l_url, out l_userName, out l_password, out l_volume);

			// Create an instance of server proxy. Cinstruct an empty header.
			ActuateAPI l_proxy = new ActuateAPI();
			l_proxy.HeaderValue = new Header();
			l_proxy.Url = l_url;
			l_proxy.HeaderValue.TargetVolume = l_volume;

			// Prepare the login reqest parameters
			Server_Proxy.localhost.Login l_req = new Server_Proxy.localhost.Login();
			l_req.Password = l_password;
			l_req.User = l_userName;
			// In case of multivolume encyclopedia, use the following element
			// to determine which volume to connect to;
			// request is forwarded to the default volume if no one specified
			// l_req.Domain = "PutYourVolumeNameHere";
			l_req.UserSetting = true;
			l_req.UserSettingSpecified = true;

			// Send reqest, handle SOAP exceptions if any
			LoginResponse l_res;
			try
			{
				l_res = l_proxy.login(l_req);
			}
			catch(Exception e)
			{
				PrintExceptionDetails(e);
				return;
			}

			//	Store the authentication id
			l_proxy.HeaderValue.AuthId = l_res.AuthId;

			//	Get admin rights value
			if(l_res.AdminRightsSpecified)	
			{
				Console.WriteLine("Administrate Right: " + l_res.AdminRights.ToString());
			}

			Console.WriteLine("User's Id: " + l_res.User.Id);
			Console.WriteLine("User's name: " + l_res.User.Name);
			Console.WriteLine("User's email address: " + l_res.User.EmailAddress);
			Console.WriteLine("User's home: " + l_res.User.HomeFolder);
		}

		///////////////////////////////////////////////////////////////////////
		//	Command line usage
		///////////////////////////////////////////////////////////////////////
		static void Usage(string[] args, out string url, out string userName, out string password, out string volume)
		{
			string l_url = "http://localhost:8000";
			string l_userName = "administrator";
			string l_password = "";
			string l_volume = "";

			for(int i = 0; i < args.Length; i++)
			{
				if(args[i].ToLower().CompareTo("-h") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_url = args[i + 1];
					}
				}
				else if(args[i].ToLower().CompareTo("-u") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_userName = args[i + 1];
					}
				}
				else if(args[i].ToLower().CompareTo("-p") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_password = args[i + 1];
					}
				}
				else if (args[i].ToLower().CompareTo("-v") == 0)
				{
				    if (!args[i + 1].StartsWith("-"))
				    {
				        l_volume = args[i + 1];
				    }
				}
			}

			url = l_url;
			userName = l_userName;
			password = l_password;
			volume = l_volume;
		}

		///////////////////////////////////////////////////////////////////////
		//	SOAP exception parsing and output
		///////////////////////////////////////////////////////////////////////
		static void PrintExceptionDetails(Exception e)
		{
			if(e is SoapException)
			{
				SoapException l_e = (SoapException)e;
				Console.WriteLine("# SOAP Fault received: ");
				for(int i = 0; i < l_e.Detail.ChildNodes.Count; i++)
				{
					if(l_e.Detail.ChildNodes.Item(i).Name[0] != '#')
					{
						if(l_e.Detail.ChildNodes.Item(i).Name == "Description")
						{
							for(int j = 0; j < l_e.Detail.ChildNodes.Item(i).ChildNodes.Count; j ++)
							{
								if(l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).Name[0] != '#')
								{
									Console.WriteLine(l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).Name + ": "
										+ l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).InnerText);
								}
							}
						}
						else
						{
							Console.WriteLine(l_e.Detail.ChildNodes.Item(i).Name + ": " + l_e.Detail.ChildNodes.Item(i).InnerText);
						}
					}
				}
			}
			else
			{
				Console.WriteLine("# Non-SOAP exception:");
				Console.WriteLine(e.ToString());
			}
		}
	}
}