/*

Actuate Client Example

This sample demonstrates SelectFiles message. The sample
selects all files in "/Report" folder prepared in Administrate
sample.

Usage:
     SelectFiles [Common options]

Example:
     SelectFiles -h http://localhost:1111 -v volume

Common options are:
     -h hostname    SOAP endpoint, default 'http://localhost:8000'
     -u username    specify user name, default 'Administrator'
     -p password    specify password, default ''
     -v volume      specify volume name, default ''
     
*/

using System;
using System.Web.Services.Protocols;
using Server_Proxy.localhost;

namespace Actuate
{
	class MainClass
	{
		[STAThread]
		static void Main(string[] args)
		{
			string l_url;
			string l_userName;
			string l_password;
			string l_volume;
			ActuateAPI l_proxy;

			Usage(args, out l_url, out l_userName, out l_password, out l_volume);
			Login(out l_proxy, l_url, l_userName, l_password, l_volume);

			//	Prepare SelectFiles request
			Server_Proxy.localhost.SelectFiles l_req = new SelectFiles();
			l_req.Item = "/Report";
            l_req.ItemElementName = ItemChoiceType53.WorkingFolderName;
			l_req.LatestVersionOnly = true;
			l_req.LatestVersionOnlySpecified = true;
			l_req.Recursive = false;
			l_req.RecursiveSpecified = true;
			l_req.ResultDef = new string[6];
			l_req.ResultDef[0] = "Id";
			l_req.ResultDef[1] = "Name";
			l_req.ResultDef[2] = "FileType";
			l_req.ResultDef[3] = "Description";
			l_req.ResultDef[4] = "Owner";
			l_req.ResultDef[5] = "Size";

			//	Prepare search criteria
			FileSearch l_fileSearch = new FileSearch();
			FileCondition l_fileCondition = new FileCondition();
			l_fileCondition.Field = FileField.Name;
			l_fileCondition.Match = "*";
			l_fileSearch.Item = l_fileCondition;
			l_fileSearch.CountLimit = -1; // count all objects in query
			l_fileSearch.CountLimitSpecified = true;

			l_req.Item1 = l_fileSearch;
			l_req.Item1ElementName = Item1ChoiceType21.Search;

			//	Send the request
			SelectFilesResponse l_res;
			try
			{
				l_res = l_proxy.selectFiles(l_req);
			}
			catch(Exception e)
			{
				PrintExceptionDetails(e);
				return;
			}

			Console.WriteLine("Total Count: " + l_res.TotalCount);

            FileArray l_fileList = (FileArray)l_res.Item;
			if (l_fileList != null)
			{
				for (int i = 0; i < l_fileList.File.Length; i++)
				{
					Console.WriteLine();
					Console.WriteLine("Item " + i + " Id:" + l_fileList.File[i].Id);
                  		Console.WriteLine("Item " + i + " Name:" + l_fileList.File[i].Name);
		                  Console.WriteLine("Item " + i + " Owner:" + l_fileList.File[i].Owner);
            		      Console.WriteLine("Item " + i + " Description:" + l_fileList.File[i].Description);
                    		Console.WriteLine("Item " + i + " File Type:" + l_fileList.File[i].FileType);
                    		Console.WriteLine("Item " + i + " File Size:" + l_fileList.File[i].Size);
				}
			}
		}

		///////////////////////////////////////////////////////////////////////
		//	Command line usage
		///////////////////////////////////////////////////////////////////////
		static void Usage(string[] args, out string url, out string userName, out string password, out string volume)
		{
			string l_url = "http://localhost:8000";
			string l_userName = "administrator";
			string l_password = "";
			string l_volume = "";

            for (int i = 0; i < args.Length; i++)
            {
                if (args[i].ToLower().CompareTo("-h") == 0)
                {
                    if (!args[i + 1].StartsWith("-"))
                    {
                        l_url = args[i + 1];
                    }
                }
                else if (args[i].ToLower().CompareTo("-u") == 0)
                {
                    if (!args[i + 1].StartsWith("-"))
                    {
                        l_userName = args[i + 1];
                    }
                }
                else if (args[i].ToLower().CompareTo("-p") == 0)
                {
                    if (!args[i + 1].StartsWith("-"))
                    {
                        l_password = args[i + 1];
                    }
                }
                else if (args[i].ToLower().CompareTo("-v") == 0)
                {
                    if (!args[i + 1].StartsWith("-"))
                    {
                        l_volume = args[i + 1];
                    }
                }
            }
			url = l_url;
			userName = l_userName;
			password = l_password;
			volume = l_volume;
		}

		///////////////////////////////////////////////////////////////////////
		//	Server login
		///////////////////////////////////////////////////////////////////////
		static void Login(out ActuateAPI proxy, string url, string userName, string password, string volume)
		{
			//	Create an instance of server proxy. Construct an empty header.
			proxy = new ActuateAPI();
			proxy.Url = url;
			proxy.HeaderValue = new Header();
			proxy.HeaderValue.TargetVolume = volume;

			//	Prepare Login reqest parameters
			Server_Proxy.localhost.Login l_LoginReq = new Server_Proxy.localhost.Login();
			l_LoginReq.User = userName;
			l_LoginReq.Password = password;

			//	Send Login reqest, handle SOAP exception if any
			LoginResponse l_loginRes;
			try
			{
				l_loginRes = proxy.login(l_LoginReq);
			}
			catch(Exception e)
			{
				Console.WriteLine(e.Message);
				return;
			}

			//	Store authentication id
			proxy.HeaderValue.AuthId = l_loginRes.AuthId;
			proxy.HeaderValue.TargetVolume = volume;
		}

		///////////////////////////////////////////////////////////////////////
		//	SOAP exception parsing and output
		///////////////////////////////////////////////////////////////////////
		static void PrintExceptionDetails(Exception e)
		{
			if(e is SoapException)
			{
				SoapException l_e = (SoapException)e;
				Console.WriteLine("# SOAP Fault received: ");
				for(int i = 0; i < l_e.Detail.ChildNodes.Count; i++)
				{
					if(l_e.Detail.ChildNodes.Item(i).Name[0] != '#')
					{
						if(l_e.Detail.ChildNodes.Item(i).Name == "Description")
						{
							for(int j = 0; j < l_e.Detail.ChildNodes.Item(i).ChildNodes.Count; j ++)
							{
								if(l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).Name[0] != '#')
								{
									Console.WriteLine(l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).Name + ": "
										+ l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).InnerText);
								}
							}
						}
						else
						{
							Console.WriteLine(l_e.Detail.ChildNodes.Item(i).Name + ": " + l_e.Detail.ChildNodes.Item(i).InnerText);
						}
					}
				}
			}
			else
			{
				Console.WriteLine("# Non-SOAP exception:");
				Console.WriteLine(e.ToString());
			}
		}
	}
}
