/*

Actuate Client Example

This sample demonstrates background report execution (asynchronous job).
Job will be scheduled to be executed in 1 minute and to be repeated daily.
The time zone is PST. The sample assumes that "/Report/SampleBIRTReport.rptdesign"
already exists in encyclopedia. Report can be uploaded to the encyclopedia
using UploadFile sample.

Usage:
     SubmitJob [Common options]

Example:
     SubmitJob -h http://localhost:1111 -v volume 

Common options are:
     -h hostname    SOAP endpoint, default 'http://localhost:8000'
     -u username    specify username, default 'Administrator'
     -p password    specify password, default ''
     -v volume      specify volume name, default ''

*/

using System;
using System.Web.Services.Protocols;
using Server_Proxy.localhost;

namespace Actuate
{
	class MainClass
	{
		[STAThread]
		static void Main(string[] args)
		{
			string l_url;
			string l_userName;
			string l_password;
			string l_volume;
			ActuateAPI l_proxy;

			Usage(args, out l_url, out l_userName, out l_password, out l_volume);
			Login(out l_proxy, l_url, l_userName, l_password, l_volume);

			// Prepare and send GetReportParameters request
			GetReportParameters l_parameter = new GetReportParameters();
            l_parameter.Item = "/Report/SampleBIRTReport.rptdesign";
            l_parameter.ItemElementName = ItemChoiceType57.ReportFileName;

			GetReportParametersResponse l_parameterRes;
			try
			{
				l_parameterRes = l_proxy.getReportParameters(l_parameter);
			}
			catch(Exception e)
			{
				PrintExceptionDetails(e);
				return;
			}

			// Prepare SubmitJob request
			Server_Proxy.localhost.SubmitJob l_req = new Server_Proxy.localhost.SubmitJob();
			l_req.JobName = "Example Job";
			l_req.Headline = "This is an example job";
			l_req.Priority = 1000;
            l_req.Item = "/Report/SampleBIRTReport.rptdesign";
            l_req.ItemElementName = ItemChoiceType63.InputFileName;
			l_req.Operation = SubmitJobOperation.RunReport;

			//	specify output file name
			l_req.RequestedOutputFile = new NewFile();
            l_req.RequestedOutputFile.Name = "/Report/SampleBIRTReport.rptdocument";

			//	prepare the job schedule
			l_req.Schedules = new JobSchedule();
			l_req.Schedules.ScheduleDetails = new JobScheduleDetail[1];
			l_req.Schedules.ScheduleDetails[0] = new JobScheduleDetail();
			l_req.Schedules.ScheduleDetails[0].ScheduleType = JobScheduleDetailScheduleType.Daily;

			Daily l_daily = new Daily();
			l_daily.FrequencyInDays = 1;
			l_daily.OnceADay = System.DateTime.Now.AddMinutes(1.0).ToString("HH:mm:ss");
			l_req.Schedules.ScheduleDetails[0].Item = l_daily;
			l_req.Schedules.TimeZoneName = "PST";

			//	Prepare the report parameters, loop through the parameter definition list
			//	retrieved from previous API call.
			if(l_parameterRes.ParameterList != null && l_parameterRes.ParameterList.Length > 0)
			{
                ParameterValueArray l_parameterValues = new ParameterValueArray();
                		l_parameterValues.ParameterValue = new ParameterValue[l_parameterRes.ParameterList.Length];
				for (int i =0; i < l_parameterRes.ParameterList.Length; i++)
				{
					l_parameterValues.ParameterValue[i] = new ParameterValue();
		                  l_parameterValues.ParameterValue[i].Name = l_parameterRes.ParameterList[i].Name;
					if(l_parameterRes.ParameterList[i].DataType == DataType.String)
					{
		                        l_parameterValues.ParameterValue[i].Value = "Oregon";					}
				}

				l_req.Item1 = l_parameterValues;
				l_req.Item1ElementName = Item1ChoiceType24.ParameterValues;
			}

			//	Send request
			SubmitJobResponse l_res;
			try
			{
				l_res = l_proxy.submitJob(l_req);
			}
			catch(Exception e)
			{
				PrintExceptionDetails(e);
				return;
			}

			Console.WriteLine("SubmitJob request was successful. Job Id: " + l_res.JobId);
		}

		///////////////////////////////////////////////////////////////////////
		//	Command line usage
		///////////////////////////////////////////////////////////////////////
		static void Usage(string[] args, out string url, out string userName, out string password, out string volume)
		{
			string l_url = "http://localhost:8000";
			string l_userName = "administrator";
			string l_password = "";
			string l_volume = "";

			for(int i = 0; i < args.Length; i++)
			{
				if(args[i].ToLower().CompareTo("-h") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_url = args[i + 1];
					}
				}
				else if(args[i].ToLower().CompareTo("-u") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_userName = args[i + 1];
					}
				}
				else if(args[i].ToLower().CompareTo("-p") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_password = args[i + 1];
					}
				}
				else if (args[i].ToLower().CompareTo("-v") == 0)
				{
				    if (!args[i + 1].StartsWith("-"))
				    {
				        l_volume = args[i + 1];
				    }
				}
			}

			url = l_url;
			userName = l_userName;
			password = l_password;
			volume = l_volume;
		}

		///////////////////////////////////////////////////////////////////////
		//	Server login
		///////////////////////////////////////////////////////////////////////
		static void Login(out ActuateAPI proxy, string url, string userName, string password, string volume)
		{
			//	Create an instance of server proxy. Construct an empty header.
			proxy = new ActuateAPI();
			proxy.Url = url;
			proxy.HeaderValue = new Header();
			proxy.HeaderValue.TargetVolume = volume;

			//	Prepare Login reqest parameters
			Server_Proxy.localhost.Login l_LoginReq = new Server_Proxy.localhost.Login();
			l_LoginReq.User = userName;
			l_LoginReq.Password = password;

			//	Send Login reqest, handle SOAP exception if any
			LoginResponse l_loginRes;
			try
			{
				l_loginRes = proxy.login(l_LoginReq);
			}
			catch(Exception e)
			{
				PrintExceptionDetails(e);
				return;
			}

			//	Store authentication id
			proxy.HeaderValue.AuthId = l_loginRes.AuthId;
		}

		///////////////////////////////////////////////////////////////////////
		//	SOAP exception parsing and output
		///////////////////////////////////////////////////////////////////////
		static void PrintExceptionDetails(Exception e)
		{
			if(e is SoapException)
			{
				SoapException l_e = (SoapException)e;
				Console.WriteLine("# SOAP Fault received: ");
				for(int i = 0; i < l_e.Detail.ChildNodes.Count; i++)
				{
					if(l_e.Detail.ChildNodes.Item(i).Name[0] != '#')
					{
						if(l_e.Detail.ChildNodes.Item(i).Name == "Description")
						{
							for(int j = 0; j < l_e.Detail.ChildNodes.Item(i).ChildNodes.Count; j ++)
							{
								if(l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).Name[0] != '#')
								{
									Console.WriteLine(l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).Name + ": "
										+ l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).InnerText);
								}
							}
						}
						else
						{
							Console.WriteLine(l_e.Detail.ChildNodes.Item(i).Name + ": " + l_e.Detail.ChildNodes.Item(i).InnerText);
						}
					}
				}
			}
			else
			{
				Console.WriteLine("# Non-SOAP exception:");
				Console.WriteLine(e.ToString());
			}
		}
	}
}
