/*

Actuate Client Example

This sample demonstrates file uploading using MIME attachments (non-chunked).
Any file can be uploaded using this sample, but "MS .NET Client/report/SampleBIRTReport.rptdesign"
is preferable since it might be used in other samples (like ExecuteReport,
SubmitJob etc).
This sample uses ActuateAPIEx class (extension) implemented in Server Proxy project
("Server Proxy.dll").

Usage:
     UploadFile [Common options] -f FileName

Example:
     UploadFile -h http://localhost:1111 -v volume -f c:\SampleBIRTReport.rptdesign

Common options are:
     -h hostname    SOAP endpoint, default 'http://localhost:8000'
     -u username    specify username, default 'Administrator'
     -p password    specify password, default ''
	 -f FileName	file to upload
     -v volume      specify volume name, default ''
     
*/

using System;
using System.Net;
using System.IO;
using System.Web.Services.Protocols;
using Server_Proxy;
using Server_Proxy.localhost;

namespace Actuate
{
	class MainClass
	{
		[STAThread]
		static void Main(string[] args)
		{
			string l_url;
			string l_userName;
			string l_password;
			string l_fileName;
			string l_volume;

			ActuateAPI l_proxy;

            Usage(args, out l_url, out l_userName, out l_password, out l_fileName, out l_volume);
			Login(out l_proxy, l_url, l_userName, l_password, l_volume);

			// Create extended proxy inctance
			ActuateAPIEx l_proxyEx= new ActuateAPIEx();
			l_proxyEx.Url = l_proxy.Url;
			l_proxyEx.HeaderValue = new Header();
			l_proxyEx.HeaderValue.AuthId = l_proxy.HeaderValue.AuthId;
			l_proxyEx.HeaderValue.TargetVolume = l_volume;

			// Prepare UploadFile request
			UploadFile l_req = new UploadFile();
			l_req.NewFile = new NewFile();
			char[] l_separators = {'/', '\\', ':'};
			string[] l_strList = l_fileName.Split(l_separators, 100);
			l_req.NewFile.Name = "/Report/" + l_strList[l_strList.Length - 1];
			l_req.Content = new Attachment();
			l_req.Content.ContentType = "binary";
			l_req.Content.ContentId = "Attachment";
			
			//	Open the file to be uploaded
			try
			{
				ActuateAPIEx.UploadStream = new FileStream(l_fileName, FileMode.Open);
			}
			catch(Exception e)
			{
				Console.WriteLine("Can not open the file" + e.Message);
				return;
			}

			// Send request
			UploadFileResponse l_res;
			try
			{
				l_res = l_proxyEx.uploadFile(l_req);
			}
			catch(Exception e)
			{
				PrintExceptionDetails(e);
			}

			ActuateAPIEx.UploadStream.Close();
		}

		///////////////////////////////////////////////////////////////////////
		//	Command line usage
		///////////////////////////////////////////////////////////////////////
        static void Usage(string[] args, out string url, out string userName, out string password, out string fileName, out string volume)
		{
			string l_url = "http://localhost:8000";
			string l_userName = "administrator";
			string l_password = "";
			string l_fileName = "";
			string l_volume = "";

			for(int i = 0; i < args.Length; i++)
			{
				if(args[i].ToLower().CompareTo("-h") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_url = args[i + 1];
					}
				}
				else if(args[i].ToLower().CompareTo("-u") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_userName = args[i + 1];
					}
				}
				else if(args[i].ToLower().CompareTo("-p") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_password = args[i + 1];
					}
				}
				else if(args[i].ToLower().CompareTo("-f") == 0)
				{
					if(!args[i + 1].StartsWith("-"))
					{
						l_fileName = args[i + 1];
					}
				}
				else if (args[i].ToLower().CompareTo("-v") == 0)
				{
				    if (!args[i + 1].StartsWith("-"))
				    {
				        l_volume = args[i + 1];
				    }
				}
            		}

			url = l_url;
			userName = l_userName;
			password = l_password;
			fileName = l_fileName;
			volume = l_volume;
        }

		///////////////////////////////////////////////////////////////////////
		//	Server login
		///////////////////////////////////////////////////////////////////////
        static void Login(out ActuateAPI proxy, string url, string userName, string password, string volume)
		{
			//	Create an instance of server proxy. Construct an empty header.
			proxy = new ActuateAPI();
			proxy.Url = url;
			proxy.HeaderValue = new Header();
			proxy.HeaderValue.TargetVolume = volume;

			//	Prepare Login reqest parameters
			Server_Proxy.localhost.Login l_LoginReq = new Server_Proxy.localhost.Login();
			l_LoginReq.User = userName;
			l_LoginReq.Password = password;

			//	Send Login reqest, handle SOAP exception if any
			LoginResponse l_loginRes;
			try
			{
				l_loginRes = proxy.login(l_LoginReq);
			}
			catch(Exception e)
			{
				PrintExceptionDetails(e);
				return;
			}

			//	Store authentication id
			proxy.HeaderValue.AuthId = l_loginRes.AuthId;
		}

		///////////////////////////////////////////////////////////////////////
		//	SOAP exception parsing and output
		///////////////////////////////////////////////////////////////////////
		static void PrintExceptionDetails(Exception e)
		{
			if(e is SoapException)
			{
				SoapException l_e = (SoapException)e;
				Console.WriteLine("# SOAP Fault received: ");
				for(int i = 0; i < l_e.Detail.ChildNodes.Count; i++)
				{
					if(l_e.Detail.ChildNodes.Item(i).Name[0] != '#')
					{
						if(l_e.Detail.ChildNodes.Item(i).Name == "Description")
						{
							for(int j = 0; j < l_e.Detail.ChildNodes.Item(i).ChildNodes.Count; j ++)
							{
								if(l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).Name[0] != '#')
								{
									Console.WriteLine(l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).Name + ": "
										+ l_e.Detail.ChildNodes.Item(i).ChildNodes.Item(j).InnerText);
								}
							}
						}
						else
						{
							Console.WriteLine(l_e.Detail.ChildNodes.Item(i).Name + ": " + l_e.Detail.ChildNodes.Item(i).InnerText);
						}
					}
				}
			}
			else
			{
				Console.WriteLine("# Non-SOAP exception:");
				Console.WriteLine(e.ToString());
			}
		}
	}
}